//-----------------------------------------------------------------------------
//
// File:	QCD4LSdll.cpp
//
// Written by Sergey Gagarin a.k.a. Seg@.
// Kirov, Russia, December 2003
//
//	This code is free.  If you redistribute it in any form, leave this notice 
//	here.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
//-----------------------------------------------------------------------------

#include "QCD4LSdll.h"
#include "stdio.h"
#include "resource.h"

HINSTANCE		hInstance;
HWND			hwndPlayer;
HWND			hwndMain;
UINT id;
QCDModInitGen	*QCDCallbacks;

BOOL CALLBACK AboutDialogHandle(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK IPCWindowHandle(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);

TCHAR szIPCWindowClass[] = "UniMedia Player Plugin Window Class";
TCHAR szIPCWindowName [] = "UniMedia Player Plugin";

bool onClose = false;

//-----------------------------------------------------------------------------

BOOL WINAPI DllMain(HINSTANCE hInst, DWORD fdwReason, LPVOID pRes)
{
	if (fdwReason == DLL_PROCESS_ATTACH)
	{
		hInstance = hInst;
	}
	return TRUE;
}

//-----------------------------------------------------------------------------

PLUGIN_API BOOL GENERALDLL_ENTRY_POINT(QCDModInitGen *ModInit, QCDModInfo *ModInfo)
{
	ModInit->version = PLUGIN_API_VERSION;
	ModInfo->moduleString = "QCD4LS v1.0";

	ModInit->toModule.ShutDown			= ShutDown;
	ModInit->toModule.About				= About;
	ModInit->toModule.Configure			= Configure;

	QCDCallbacks = ModInit;

	hwndPlayer = (HWND)QCDCallbacks->Service(opGetParentWnd, 0, 0, 0);

	char inifile[MAX_PATH];
	QCDCallbacks->Service(opGetPluginSettingsFile, inifile, MAX_PATH, 0);


	//own initialization

    // Create the window
	WNDCLASSEX wc;
	ZeroMemory(&wc, sizeof(WNDCLASSEX));
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.hInstance = hInstance;
	wc.style = CS_DBLCLKS;
	wc.lpfnWndProc = IPCWindowHandle; 
	wc.lpszClassName = szIPCWindowClass;
    
    if (!RegisterClassEx(&wc)) 
	{ 
		MessageBox(NULL, "Error: Could not register window class!",
            "QCD4LS", MB_OK | MB_ICONERROR | MB_TOPMOST | MB_SETFOREGROUND); 
		
        return false;
	}

    hwndMain = CreateWindowEx(WS_EX_TOOLWINDOW, szIPCWindowClass, szIPCWindowName,
                              WS_POPUP,
                              0, 0,
                              10, 10,
							  hwndPlayer,
                              NULL, hInstance, NULL);
	if (!hwndMain)
	{
		MessageBox(NULL, "Error: Could not create main window!",
            "QCD4LS", MB_OK | MB_ICONERROR | MB_TOPMOST | MB_SETFOREGROUND); 
		return false;
	}
	else
	{
		id = SetTimer(hwndMain,1,50,NULL);
		ShowWindow(hwndMain, SW_HIDE);
	}

	// return TRUE for successful initialization
	return TRUE;
}

//----------------------------------------------------------------------------

void ShutDown(int flags) 
{
	onClose = true;
	HWND AboutDialog = FindWindow(NULL,"About QCD4LS...");
	if (AboutDialog)
		DestroyWindow(AboutDialog);
	KillTimer(hwndMain, id);
}

//-----------------------------------------------------------------------------

void Configure(int flags)
{
	MessageBox(hwndPlayer,"Errr... You need to config something?.. What to config?!","There is nothing to setup!",0);
}

//-----------------------------------------------------------------------------

void About(int flags)
{
	HWND AboutDialog = FindWindow(NULL,"About QCD4LS...");
	if (AboutDialog)
	{
		SetForegroundWindow(AboutDialog);
	}
	else
	{
		AboutDialog = CreateDialog(hInstance,MAKEINTRESOURCE(IDD_ABOUTDIALOG),hwndPlayer,AboutDialogHandle);
		ShowWindow(AboutDialog,SW_SHOW);
		UpdateWindow(AboutDialog);	
	}
}

//-----------------------------------------------------------------------------

BOOL CALLBACK AboutDialogHandle(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
	case WM_COMMAND:
		{
			switch(LOWORD(wParam))
			{
			case IDOK:
				DestroyWindow(hwndDlg);
				return TRUE;
			}
			break;
		}
	case WM_INITDIALOG:
		return TRUE;
	}
	return FALSE;
}

//-----------------------------------------------------------------------------

LRESULT CALLBACK IPCWindowHandle(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	char buf[1024];
	if (!onClose)
	switch (msg)
	{
	case WM_TIMER:
		sprintf(buf,"%d. ",QCDCallbacks->Service(opGetCurrentIndex,NULL,0,0)+1);
		if (QCDCallbacks->Service(opGetTrackName,buf+strlen(buf),512,-1))
			strcat(buf," - Winamp");
		else
			strcpy(buf,"");
		SetWindowText(hwnd,buf);
		break;
	case WM_USER:
		if (wParam == 1 && lParam == 105)	//length
			return QCDCallbacks->Service(opGetTrackLength,NULL,-1,0);
		if (wParam == 0 && lParam == 105)	//position
			return QCDCallbacks->Service(opGetTime,NULL,1,0) * 1000;
		if (wParam == 1 && lParam == 126)	//bitrate
			return QCDCallbacks->Service(opGetAudioInfo,NULL,1,0) / 1000;
		if (wParam == 0 && lParam == 104)	//status
		{
			int st = QCDCallbacks->Service(opGetPlayerState,NULL,0,0);
			if (st == 1 || st == 2) st = 3 - st;
			return st;
		}
		break;
    }
	return DefWindowProc (hwnd, msg, wParam, lParam);
}

//-----------------------------------------------------------------------------
